const cheerio = require('cheerio');
const { ObjectID } = require('mongodb');

const IgrMasterServices = require('../Services/CreateServices')("IgrMaster");
const IgrDumpServices = require('../Services/CreateServices')("IgrDump");
const IgrIndexServices = require('../Services/CreateServices')("IgrIndex");

const Config = require('../Constants/Config');
const IgrDumpESServices = require('../Services/CreateESServices')(Config.ES_IGR_DUMP_INDEX);

module.exports.ESfindIGRMasters = async (req, res) => {
    try {
        let { type = "year", year, district, village: area, DName, searchText } = req.query;
        if (type == "district") { type = "districtName" }

        let aggs = {
            [`${type}`]: {
                terms: {
                    field: `${type}.keyword`,
                    "size": 1000,
                    "order": {
                        "_key": "desc" // Sort by date histogram key in descending order
                    }
                }
            }
        };
        let query = { bool: { must: [] } };

        let headers = [];


        const igrMasterRes = await IgrDumpESServices.aggregate({
            'index': Config.ES_IGR_DUMP_INDEX,
            "size": 0,
            "track_total_hits": true,
            "query": query,
            "aggs": aggs
        });
        if (!igrMasterRes.status || igrMasterRes.error) return res.status(500).send(igrMasterRes);

        let data = [];


        console.log("-> res", igrMasterRes.data[type], type);
        data = igrMasterRes.data[type]?.buckets.map(bucket => {
            const key = bucket.key;
            const count = bucket.doc_count
            //if (type == "districtName") { type = "district" }
            return {
                [type]: key,
                count
            };
        });


        return res.status(200).send({ status: true, data, headers });
    } catch (e) {
        console.log("🚀 ~ file: IgrMaster.controller.js:39 ~ module.exports.findIGRMasters= ~ e:", e)
        return res.status(500).send({ status: false, error: e.toString() });
    }
}

module.exports.ESfindIGRMastersOld = async (req, res) => {
    try {
        const { type, year, district, village: area, DName, searchText } = req.query;
        let aggs = {
            year: {
                terms: {
                    field: 'year.keyword',
                    "size": 1000,
                    "order": {
                        "_key": "desc" // Sort by date histogram key in descending order
                    }
                }
            }
        };
        let query = { bool: { must: [] } };

        let headers = [];

        if (!!year) {
            query.bool.must.push({ term: { "year.keyword": year } });
            aggs = {
                district: {
                    terms: { field: 'district.keyword', "size": 1000 },
                    aggs: {
                        districtName: {
                            terms: { field: "districtName.keyword", "size": 1 }
                        }
                    }
                }
            }
        }
        if (!!district) {
            query.bool.must.push({ term: { "district.keyword": district } });
            aggs = { area: { terms: { field: "area.keyword", "size": 1000 } } }
        }
        if (!!area) {
            query.bool.must.push({ term: { "area.keyword": area } });
            aggs = { DName: { terms: { field: "DName.keyword", "size": 1000 } } }
        }
        if (!!DName) {
            query.bool.must.push({ term: { "DName.keyword": DName } });
        }

        const igrMasterRes = await IgrDumpESServices.aggregate({
            'index': Config.ES_IGR_DUMP_INDEX,
            "size": 0,
            "track_total_hits": true,
            "query": query,
            "aggs": aggs
        });
        if (!igrMasterRes.status || igrMasterRes.error) return res.status(500).send(igrMasterRes);

        let data = [];

        if (type === "district") {
            data = igrMasterRes.data.district.buckets.map(bucket => {
                const districtNameBuckets = bucket.districtName.buckets[0]
                return {
                    count: bucket.doc_count,
                    district: bucket.key,
                    districtName: districtNameBuckets['key']
                };
            });
        } else {
            console.log("-> res", igrMasterRes.data[type], type);
            data = igrMasterRes.data[type]?.buckets.map(bucket => {
                const key = bucket.key;
                const count = bucket.doc_count

                return {
                    [type]: key,
                    count
                };
            });
        }

        return res.status(200).send({ status: true, data, headers });
    } catch (e) {
        console.log("🚀 ~ file: IgrMaster.controller.js:39 ~ module.exports.findIGRMasters= ~ e:", e)
        return res.status(500).send({ status: false, error: e });
    }
}

module.exports.ESIgrSearch = async (req, res) => {
    try {
        const { type, year, district, area, DName, searchText } = req.query;
        const { startIndex, limit } = req.local;
        let query = { "bool": { "must": [] } };
        let headers = [];

        if (!!year) { query.bool.must.push({ term: { "year.keyword": year } }); }
        if (!!district) { query.bool.must.push({ term: { "districtName.keyword": district } }); }
        if (!!area) { query.bool.must.push({ term: { "area.keyword": area } }); }
        if (!!DName) { query.bool.must.push({ term: { "DName.keyword": DName } }); }

        if (!!searchText) {
            query.bool.must.push({
                "bool": {
                    "should": [
                        {
                            "bool": {
                                "must": [
                                    ...searchText.split(',').map((ss, sid) => {
                                        return { "match_phrase": { "Property Description": ss } }
                                    }),
                                ]
                            }
                        },
                        {
                            "match_phrase": {
                                "Seller Name": searchText
                            }
                        },
                        {
                            "match_phrase": {
                                "Purchaser Name": searchText
                            }
                        }
                    ]
                }
            });
        }

        const igrMasterRes = await IgrDumpESServices.find({ query: query, sort: [{ "RDate": { order: 'desc' } }], display: ["_id", "area", "tahsil", "district", "DocNo", "DName", "RDate", "SROName", "Seller Name", "Purchaser Name", "Property Description", "SROCode", "Status", "IndexII", "content", "location", "indicator"], start: startIndex, limit: limit });
        if (!igrMasterRes.status || !!igrMasterRes.error) return res.status(500).send(igrMasterRes);

        return res.status(200).send({ ...igrMasterRes, headers, query });
    } catch (e) {
        logger.error("🚀 ~ file: IgrMaster.controller.js:69 ~ module.exports.findIGRMasters= ~ e:", e)
        return res.status(500).send({ status: false, error: e });
    }
}

module.exports.findMasters = async (req, res) => {
    try {
        const { type, year, district, village: area, DName, searchText } = req.query;
        let group = { year: '$year' };
        let query = {};
        let project = { _id: 0, data: 1, count: 1 };

        let headers = [];

        if (!!year) { query.year = year; group.district = '$district'; group.districtName = '$districtName' }
        if (!!district) { query.district = district; group.area = '$area' }
        if (!!area) { query.area = area; group.DName = '$DName' }
        if (!!DName) { query.DName = DName; }

        Object.keys(group).map((k, id) => { project[k] = `$_id.${k}`; })

        const igrMasterRes = await IgrMasterServices.aggregate([
            { '$match': query },
            {
                '$group': {
                    _id: { ...group },
                    count: { $sum: 1 }
                }
            },
            { '$project': project }
        ]);
        if (!igrMasterRes.status || igrMasterRes.error) return res.status(500).send(igrMasterRes);

        return res.status(200).send({ ...igrMasterRes, headers });
    } catch (e) {
        console.log("🚀 ~ file: IgrMaster.controller.js:39 ~ module.exports.findIGRMasters= ~ e:", e)
        return res.status(500).send({ status: false, error: e });
    }
}

module.exports.findIGRMasters = async (req, res) => {
    try {
        const { type, year, district, village: area, DName, searchText } = req.query;
        let group = { year: '$year' };
        let query = {};
        let project = { _id: 0, data: 1, count: 1 };

        let headers = [];

        if (!!year) { query.year = year; group.district = '$district'; group.districtName = '$districtName' }
        if (!!district) { query.district = district; group.area = '$area' }
        if (!!area) { query.area = area; group.DName = '$DName' }
        if (!!DName) { query.DName = DName; }

        Object.keys(group).map((k, id) => { project[k] = `$_id.${k}`; })

        const igrMasterRes = await IgrDumpServices.aggregate([
            { '$match': query },
            {
                '$group': {
                    _id: { ...group },
                    count: { $sum: 1 }
                }
            },
            { '$project': project }
        ]);
        if (!igrMasterRes.status || igrMasterRes.error) return res.status(500).send(igrMasterRes);

        return res.status(200).send({ ...igrMasterRes, headers });
    } catch (e) {
        console.log("🚀 ~ file: IgrMaster.controller.js:39 ~ module.exports.findIGRMasters= ~ e:", e)
        return res.status(500).send({ status: false, error: e });
    }
}

module.exports.IgrSearch = async (req, res) => {
    try {
        const { type, locationAreaType, tahsil, year, district, area, DName, searchText } = req.query;
        const { startIndex, limit } = req.local;
        let query = {};
        let headers = [];

        if (!!locationAreaType) { query.locationAreaType = locationAreaType; }
        if (!!year) { query.year = year; }
        if (!!district) { query.district = district; }
        if (!!tahsil) { query.tahsil = tahsil; }
        if (!!area) { query.area = area; }
        if (!!DName) { query.DName = { $in: DName.split(',').map((d, id) => d.trim()) } }
        if (!!searchText) {
            query['$or'] = [
                { 'DocNo': { $regex: searchText, $options: 'i' } },
                { 'Seller Name': { $regex: searchText, $options: 'i' } },
                { 'Purchaser Name': { $regex: searchText, $options: 'i' } },
                { 'Property Description': { $regex: searchText, $options: 'i' } },
            ]
        }

        const igrMasterRes = await IgrDumpServices.find({ query: query, sort: { RDate: -1 }, display: { area: 1, tahsil: 1, district: 1, DocNo: 1, DName: 1, RDate: 1, SROName: 1, 'Seller Name': 1, 'Purchaser Name': 1, 'Property Description': 1, 'SROCode': 1, 'Status': 1, IndexII: 1, 'content': 1, location: 1, indicator: 1, }, start: startIndex, limit: limit });
        if (!igrMasterRes.status || igrMasterRes.error) return res.status(500).send(igrMasterRes);

        return res.status(200).send({ ...igrMasterRes, headers });
    } catch (e) {
        console.log("🚀 ~ file: IgrMaster.controller.js:69 ~ module.exports.findIGRMasters= ~ e:", e)
        return res.status(500).send({ status: false, error: e });
    }
}

module.exports.getIgr = async (req, res) => {
    try {
        const id = req.params.id;
        // change elasticsearch get query for 1 document
        const IgrDumpRes = await IgrIndexServices.findById(id);
        if (!IgrDumpRes.status || IgrDumpRes.error) return res.status(500).send(IgrDumpRes);
        if (IgrDumpRes.data.length == 0) return res.status(404).send({ status: true, message: 'No data available' });

        const igr = IgrDumpRes.data;

        const printCode = `<script>
        let title = document.createElement('title');
        title.innerHTML = '${`igr_${igr?.DocNo}`}';
        document.getElementsByTagName('head')[0].appendChild(title);
        setTimeout(function () {
                window.print({
                    headers: false,
                    footers: false
                });
            }, 500);
        window.onafterprint = function(){ window.close()}; 
        </script>`;

        let content = igr.content + printCode;
        content = content.replace('<br><br><br><br><br><br>\n\n\n\n\n', '');

        const resContent = cheerio.load(content);
        resContent('#form1').remove();
        resContent('<br>').remove();
        return res.status(200).send(resContent.html());
    } catch (e) {
        console.log("🚀 ~ file: MasterData.controller.js:25 ~ module.exports.createMasterData= ~ e:", e)
        return res.status(500).send({ status: false, error: e });
    }
}

module.exports.IgrIndexDetails = async (req, res) => {
    try {
        const id = req.params.id;
        // change elasticsearch query
        const IgrIndexRes = await IgrIndexServices.findById(id);
        if (!IgrIndexRes.status || IgrIndexRes.error) return res.status(500).send(IgrIndexRes);
        if (IgrIndexRes.data.length == 0) return res.status(404).send({ status: true, message: 'No data available' });

        const igr = IgrIndexRes.data;
        return res.status(200).send(igr);
    } catch (e) {
        console.log("🚀 ~ file: Igrs.controller.js:477 ~ module.exports.igrindexdetails= ~ e:", e)
        return res.status(500).send({ status: false, error: e });
    }
}
