const Payment = require('../Model/Payment');
const defaultUserSelect = 'firstName lastName email profilePhoto';
const defaultPopulate = [
    { path: 'requestedBy', select: defaultUserSelect },
    { path: 'collectedBy', select: defaultUserSelect },
    { path: 'processedBy', select: defaultUserSelect },
]

/**
 * @description This is used for get All cases
 * @param {*} query 
 * @param {*} displayFields 
 * @param {*} startIndex 
 * @param {*} limit 
 * @returns Object { status: true, cases: Array }
 *  { status: false, error: String }
 */
module.exports.getAllPayments = async (query = {}, { display = [], sort = { createdAt: -1 }, populate = [] } = {}, limit, startIndex) => {

    return Payment.find(query, display).populate([...defaultPopulate, ...populate]).sort(sort).skip(startIndex).limit(limit)
        .then((payments) => {
            return { status: true, payments: payments };
        }).catch((e) => {
            return { status: false, error: e };
        });
};

/**
 * @description This is used for get One payment By Id 
 * @param {String} id 
 * @returns Object { status: true, payments: Object }
 * - { status: false, error: string }
 */
module.exports.getPayment = async (id, { populate } = { populate: [] }) => {

    return Payment.findById(id).populate([...populate, ...defaultPopulate]).then((payments) => {
        return { status: true, payments: payments };
    }).catch((e) => {
        return { status: false, error: e };
    });
};

module.exports.aggregate = async (query) => {
    return Payment.aggregate(query)
        .then((payments) => {
            return { status: true, payments: payments };
        }).catch((e) => {
            console.log(e);
            return { status: false, error: e };
        });
};

module.exports.count = async (query) => {
    return Payment.countDocuments(query)
        .then((payments) => {
            return { status: true, count: payments };
        }).catch((e) => {
            console.log(e);
            return { status: false, error: e };
        });
};

/**
 * @description This is used for  create Payment 
 * @param {*} PaymentData 
 * @returns  Object { status: true, payments: Object }
 * - { status: false, error: string }
 */
module.exports.createPayment = async (PaymentData) => {

    return Payment.create(PaymentData)
        .then((result) => { return { status: true, payments: result }; })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })
};

/**
 * @description  This is used for update Payment 
 * @param {String} id 
 * @param {Object} PaymentData 
 * @returns  Object { status: true, payments: Object }
 * - { status: false, error: string }
 */
module.exports.updatePayment = async (id, PaymentData) => {

    return Payment.findOneAndUpdate({ _id: id }, PaymentData, { new: true })
        .then((result) => { return { status: true, payments: result }; })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })
};

/**
 * @description This is used for update payment from query
 * @param {Object} find 
 * @param {Object} PaymentData 
 * @returns Object { status: true, payments: Object }
 * - { status: false, error: string }
 */
module.exports.findUpdatePayment = async (find, PaymentData) => {

    return Payment.findOneAndUpdate(find, PaymentData, { new: true })
        .then((result) => { return { status: true, payments: result }; })
        .catch((err) => {
            console.log("🚀 ~ file: Payment.services.js:80 ~ module.exports.findUpdatePayment= ~ err:", err)
            return { status: false, error: err }
        })
};

/**
 * @description This is used for update Many payments
 * @param {Object} find_query 
 * @param {Object}  PaymentData
 * @returns Object { status: true, payments: result }
 * -  { status: false, err }
 */
module.exports.updateMultiplePayment = async (id, update, filter) => {
    return Payment.updateMany(id, update, filter)
        .then((result) => { return { status: true, payments: result } })
        .catch((err) => {
            console.log(err)
            return { status: false, err }
        })
};

/**
 * @description  This is used for delete Payment 
 * @param {String} id 
 * @returns  Object { status: true, payments: Object }
 * - { status: false, error: string }
 */
module.exports.deletePayment = async (query) => {
    return Payment.deleteOne(query)
        .then((result) => { return { status: true, payments: result }; })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })
};
