const express = require('express');
const router = express.Router();
const CaseController = require('../Controllers/Case.controller');
const UtilitiesController = require('../Controllers/Utilities.controller');
const isAuth = require('../Middleware/isAuth');
const isAdmin = require('../Middleware/isAdmin');
const validation = require('../Middleware/validation');
const uploadFile = require('../Middleware/uploadFile');
const isPermission = require('../Middleware/isPermission');
const pagination = require('../Middleware/pagination');
const { FIELD_HEADERS } = require('../Types/LandBuilding');

// For display Nodes
router.get('/dashboard', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "view" }) }, CaseController.dashboard);

// For List of Cases
router.get('/cases', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "list" }) }, pagination, CaseController.getAllCases);

// For get One Case
router.get('/cases/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "view" }) }, CaseController.getCase);

// For Verified Documents of Lead
router.post('/cases/:id/verifiedDocuments', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "lead", value: "edit" }) }, CaseController.verifiedDocuments);

// For create Case
router.post('/cases', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "create" }) }, validation.caseValidation, CaseController.createCase);

// For update Case
router.put('/cases/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "edit" }) }, validation.caseValidation, CaseController.updateCase);

// For update case Status i.e. reflect in Nodes.
router.post('/cases/:id/caseStatus', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "edit" }) }, CaseController.updateCaseStatus);

router.get('/caseLogList/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "list" }) }, CaseController.getCaseLog);

// For update Site Visit Status
router.post('/cases/updateSitevisit/:id', isAuth, CaseController.rescheduleSiteVisit);

// For Replace key into Report Template key
router.post('/case/:id/replaceData', isAuth, CaseController.replaceData);
router.post('/case/:id/generate-report', isAuth, CaseController.replaceData);
router.put('/case/:id/update-dates', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "edit" }) }, validation.caseUpdateDateValidation, CaseController.updateDates)

// For get near by completed cases into  1 km Range.
router.get('/nearByLocation', CaseController.nearByLocation);
router.get('/nearByLocation/:id', CaseController.nearByLocation);
router.get('/searchByAddress', CaseController.searchByAddress);

// For calculate Fees from Fees Structure
router.post('/feeCalculate', isAuth, CaseController.feeCalculate);

//UPLOAD external report details to display on map
router.post('/case/:id/external-report-details', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "edit" }) }, validation.caseExternalDetailsValidation, CaseController.externalReportDetails)

// UPLOAD external report
router.post('/case/:id/external-document-image', isAuth, uploadFile.uploadExternalDocumentImage, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "edit" }) }, CaseController.externalDocumentImageUpdate)
router.post('/case/:id/external-document', isAuth, uploadFile.uploadExternalDocuments, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "edit" }) }, CaseController.externalDocumentUpdate)
router.delete('/case/:id/external-document', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "edit" }) }, CaseController.externalDocumentDelete)

// UPLOAD Processing documents
router.post('/case/:id/processing-document', isAuth, uploadFile.uploadProcessingDocuments, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "edit" }) }, CaseController.processingDocumentUpdate)
router.delete('/case/:id/processing-document', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "edit" }) }, CaseController.processingDocumentDelete)

router.post('/saveOldOutward/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "edit" }) }, CaseController.saveOldOutward)
router.post('/generateOutward/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "edit" }) }, CaseController.generateOutward)

router.put('/cases/:id/remarks', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "edit" }) }, validation.addCaseRemark, CaseController.addCaseRemark);

// APP Routes

// For update Case Valuation for SITE ENGINEER MOBILE APP
router.put('/cases/:id/caseValuation', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "edit" }) }, validation.caseValuationValidation, CaseController.updateCaseValuation);
router.put('/cases/:id/casePartialInspection', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "edit" }) }, validation.casePartialInspectionValidation, CaseController.sitePartialInspection);

router.put('/cases/:id/approxValuation', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "edit" }) }, validation.caseValuationValidation, CaseController.approxCaseValuation);

// For List of Cases
router.get('/getAllCases', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "list" }) }, pagination, CaseController.getAllPendingCases);

router.get('/case-lead-search', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "list" }) }, pagination, CaseController.getLeadCaseList);

router.get('/landAndBuildingTypes', (req, res, next) => {
    return res.status(200).send({ status: true, headers: FIELD_HEADERS });
});

router.post('/case-copy/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "create" }) }, CaseController.copyCase)

module.exports = router;